import { Component, OnInit } from "@angular/core";
import { FormBuilder, FormGroup, Validators } from "@angular/forms";
import { ActivatedRoute } from "@angular/router";
import { NzMessageService } from "ng-zorro-antd";
import { Subject } from "rxjs";
import { debounceTime } from "rxjs/operators";
import { MainService } from "src/app/services/main.service";
import { ToolService } from "src/app/services/tool.service";

@Component({
  selector: "app-hospital-config",
  templateUrl: "./hospital-config.component.html",
  styleUrls: ["./hospital-config.component.less"],
})
export class HospitalConfigComponent implements OnInit {
  saveLoading = false;
  hosId;
  validateForm: FormGroup;
  isLoading: boolean = false;
  searchLoading: boolean = false;
  hospitalConfigList = []; //院区系统配置列表
  roleList = []; //角色列表
  taskTypeList = []; //任务类型列表
  deptList = []; //科室列表
  groupList = []; //组列表
  onSearchSubject = new Subject(); //搜索防抖
  coopBtns = {};
  constructor(
    private mainService: MainService,
    private fb: FormBuilder,
    private route: ActivatedRoute,
    private tool: ToolService,
    private message: NzMessageService
  ) {}

  ngOnInit() {
    this.hosId = this.tool.getCurrentHospital().id;
    this.onSearchSubject.pipe(debounceTime(500)).subscribe((v) => {
      this.getDeptList(this.hosId, v).then((result) => {
        this.searchLoading = false;
        if (result.status == 200) {
          this.deptList = result.list;
        }
      });
    });
    this.coopBtns = this.tool.initCoopBtns(this.route);
    this.getInit();
  }
  // 根据科室id查科室
  getDeptById(id) {
    return this.mainService.getFetchData("data", "department", id);
  }
  //初始化请求数据
  getInit() {
    this.isLoading = true;
    Promise.all([
      this.getRoleList(),
      this.getDeptList(this.hosId),
      this.getGroupList(this.hosId),
      this.getTaskTypeList([255]),
      this.getClassesList(),
    ])
      .then((result) => {
        if (result[0].status == 200) {
          this.roleList = result[0].list;
        }
        if (result[1].status == 200) {
          this.deptList = result[1].list;
        }
        if (result[2].status == 200) {
          this.groupList = result[2].list;
        }
        if (result[3].status == 200) {
          this.taskTypeList = result[3].list;
        }
        if (result[4].status == 200) {
          this.classesList = result[4].list;
          this.classesList = this.classesList.map((item) => ({
            label: item.name,
            value: item.id,
            checked: false,
          }));
        }
        this.getHospitalConfigList(this.hosId).subscribe((res) => {
          if (res.status == 200) {
            this.hospitalConfigList = res.list;
            //创建表单
            let fbGroup = {};
            this.hospitalConfigList.forEach((item) => {
              if (item.key.includes("phone_num_port")) {
                //坐席号码
                fbGroup[item.key] = [null];
                item.required = false;
              } else if (item.key === "rushNoticeShift") {
                //抢单通知班次
                fbGroup[item.key] = [];
                item.required = false;
              } else {
                fbGroup[item.key] = [null, [Validators.required]];
                item.required = true;
              }
            });
            this.validateForm = this.fb.group(fbGroup);
            //赋值
            this.hospitalConfigList.forEach((config) => {
              //默认角色,默认组
              if (
                config.key === "autoCreateUserRoleId" ||
                config.key === "rebackPatientTypeId" ||
                config.key === "transDeptTypeId" ||
                config.key === "autoCreateUserGroupId"
              ) {
                this.validateForm.controls[config.key].setValue(
                  config.value - 0
                );
              } else if (config.key === "autoCreateUserDeptId") {
                //默认科室
                let flag = this.deptList.some(
                  (item) => item.id == config.value
                );
                console.log(this.deptList);
                if (!flag) {
                  this.getDeptById(config.value).subscribe((item) => {
                    this.isLoading = false;
                    if (item.status == 200) {
                      if (item.data.hospital.id == this.hosId) {
                        console.log(this.deptList);
                        this.deptList.unshift(item.data);
                      }
                    }
                  });
                } else {
                  this.isLoading = false;
                }
                this.validateForm.controls[config.key].setValue(
                  config.value - 0
                );
              } else if (config.key === "rushNoticeShift") {
                let arr = config.value ? config.value.split(",") : [];
                this.classesList.forEach((item) => {
                  item.checked = arr.includes(item.value + "");
                });
                this.validateForm.controls[config.key].setValue(
                  this.classesList
                );
              } else {
                this.validateForm.controls[config.key].setValue(config.value);
              }
            });
          } else {
            this.isLoading = false;
          }
        });
      })
      .catch((err) => {
        this.isLoading = false;
      });
  }
  // 获取当前院区的班次列表
  classesList = [];
  getClassesList() {
    let postData = {
      idx: 0,
      sum: 9999,
      scheduleClass: {
        hospital: { id: this.hosId },
      },
    };
    return this.mainService
      .getFetchDataList("configuration", "scheduleClass", postData)
      .toPromise();
  }
  // 获取系统配置列表
  getHospitalConfigList(hosId) {
    let postData = { idx: 0, sum: 100, hospitalConfig: { hosId } };
    return this.mainService.getFetchDataList(
      "simple/data",
      "hospitalConfig",
      postData
    );
  }
  // 获取角色列表
  getRoleList() {
    let postData = { idx: 0, sum: 100 };
    return this.mainService
      .getFetchDataList("user/data", "role", postData)
      .toPromise();
  }
  // 获取任务类型列表(患者其他服务)
  getTaskTypeList(associationTypeIds?) {
    let postData: any = {
      idx: 0,
      sum: 999,
      taskType: {
        hosIds: this.hosId,
        simpleQuery: true,
      },
    };
    if (associationTypeIds) {
      postData.taskType.associationTypeIds = associationTypeIds.toString();
    }
    return this.mainService
      .getFetchDataList("configuration", "taskType", postData)
      .toPromise();
  }
  /**
   * 获取科室列表
   * @param hosId 院区id
   * @param dept 模糊搜索
   * @returns
   */
  getDeptList(hosId, dept: any = "") {
    let postData = {
      idx: 0,
      sum: 10,
      department: {
        hospital: { id: hosId },
        dept,
      },
    };
    return this.mainService
      .getFetchDataList("data", "department", postData)
      .toPromise();
  }
  /**
   * 获取组列表
   * @param hosId 院区id
   * @returns
   */
  getGroupList(hosId) {
    let postData = { idx: 0, sum: 100, group2: { hospitals: hosId } };
    return this.mainService
      .getFetchDataList("data", "group2", postData)
      .toPromise();
  }
  //搜索
  onSearch(e) {
    this.searchLoading = true;
    this.onSearchSubject.next(e);
  }

  // 保存
  submitForm(): void {
    for (const i in this.validateForm.controls) {
      this.validateForm.controls[i].markAsDirty();
      this.validateForm.controls[i].updateValueAndValidity();
    }
    if (this.validateForm.invalid) return;
    let hospitalConfigList = JSON.parse(
      JSON.stringify(this.hospitalConfigList)
    );
    hospitalConfigList.forEach((config) => {
      if (config.key === "rushNoticeShift") {
        config.value = this.validateForm.controls[config.key].value
          .filter((item) => item.checked)
          .map((item) => item.value)
          .toString();
      } else {
        config.value = this.validateForm.controls[config.key].value;
      }
      delete config.required;
    });
    this.saveLoading = true;
    this.mainService.changeHospitalSysConfig(hospitalConfigList).subscribe(
      (result) => {
        this.saveLoading = false;
        if (result["status"] == 200) {
          this.message.create("success", "保存成功");
          this.getInit();
        } else {
          this.message.create("error", "保存失败");
        }
      },
      (error) => {
        this.saveLoading = false;
        this.message.create("error", "保存失败");
      }
    );
  }
}
